package cn.iocoder.yudao.module.system.job.equipment;

import cn.iocoder.yudao.framework.tenant.core.job.TenantJob;
import cn.iocoder.yudao.framework.websocket.core.session.WebSocketSessionManager;
import cn.iocoder.yudao.module.system.dal.dataobject.equipment.AttendanceMachineDO;
import cn.iocoder.yudao.module.system.service.equipment.AttendanceMachineService;
import com.github.yulichang.toolkit.SpringContentUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

@Component
@Slf4j
@EnableScheduling
public class AttendanceMachineJob {

    @Resource
    private AttendanceMachineService attendanceMachineService;

    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Scheduled(initialDelay = 5000, fixedRate = 30000)
    @TenantJob
    public void updateAttendanceMachineStatus() {

        // 获得所有在线的设备
        List<AttendanceMachineDO> attendanceMachineDOS = attendanceMachineService.getListByStatus();
        for (AttendanceMachineDO reqDO : attendanceMachineDOS) {

            // redis数据不存在，则为离线状态
            if (stringRedisTemplate.opsForValue().get(reqDO.getDeviceNo()) == null) {

                attendanceMachineService.updateAttendanceMachineStatus(reqDO.getDeviceNo(), 0, null);

                //同步 删除对应sessionId
                WebSocketSessionManager webSocketSessionManager = SpringContentUtils.getBean(WebSocketSessionManager.class);
                webSocketSessionManager.removeSession(webSocketSessionManager.getSessionByDeviceNum(reqDO.getDeviceNo()));
            }else {

                attendanceMachineService.updateAttendanceMachineStatus(reqDO.getDeviceNo(), 1, stringRedisTemplate.opsForValue().get(reqDO.getDeviceNo()));
            }
        }
    }
}
